:BasicUpstart2(main)

.label cia1_interrupt_control_register = $dc0d
.label cia2_interrupt_control_register = $dd0d

.label vic2_interrupt_control_register = $d01a
.label vic2_interrupt_status_register = $d019
.label vic2_rasterline_register = $d012
.label screen_control_register = $d011

.label border_color = $d020
.label background_color = $d021

.label irq_vector = $0314
.label irq_handler = $ea31

main:
  :background(DARK_GRAY)
  :border(GRAY)
  // Disable IRQ handling
  sei 

  // Disable the interrupt generated by timers
  lda #%01111111
  sta cia1_interrupt_control_register
  sta cia2_interrupt_control_register  

  // Negate already scheduled interrupts 
  lda cia1_interrupt_control_register
  lda cia2_interrupt_control_register

  // enable raster interrupts
  lda #%00000001
  sta vic2_interrupt_control_register

  :request_raster_irq(106, raster_bars_irq)

  // Enable IRQ handling
  cli

  rts

raster_bars_irq:  

  :background(BLACK)
  :nops(230)
  :background(DARK_GRAY)

  :exit_irq_handler()

.macro border(color) {
  lda #color
  sta border_color
}
.macro background(color) {
  lda #color
  sta background_color
}
.macro nops(count) {
  .for (var i = 0; i < count; i++) {
    nop
  } 
}

.macro set_bits(address, bit_mask) {
  lda address
  ora #bit_mask
  sta address
}
.macro clear_bits(address, bit_mask) {
  lda address
  and #255 - bit_mask
  sta address
}

.macro request_raster_irq(line_number, handler) {
  lda #line_number 
  sta vic2_rasterline_register
  
  .if (line_number > 255) {
    :set_bits(screen_control_register, %10000000)
  } else {
    :clear_bits(screen_control_register, %10000000)
  }

  lda #<handler
  sta irq_vector
  lda #>handler
  sta irq_vector + 1
}

.macro exit_irq_handler() {
  lda #%00000001
  sta vic2_interrupt_status_register
  jmp irq_handler
}

.pc = $3fff "Border pattern"
.byte %11111111
