:BasicUpstart2(main)

.label cpu_io_port = $01

.label chrout = $ffd2

.label cia1_interrupt_control_register = $dc0d
.label cia2_interrupt_control_register = $dd0d

.label vic2_interrupt_control_register = $d01a
.label vic2_interrupt_status_register = $d019
.label vic2_rasterline_register = $d012
.label screen_control_register = $d011

.label border_color = $d020
.label background_color = $d021

.label irq_vector = $0314
.label irq_handler = $ea31

.label screen_memory = $0400

.const VIC2 = $d000
.namespace sprites {
  .label positions = VIC2
  .label position_x_high_bits = VIC2 + 16
  .label enable_bits = VIC2 + 21
  .label colors = VIC2 + 39
  .label vertical_stretch_bits = VIC2 + 23
  .label horizontal_stretch_bits = VIC2 + 29
  .label priority_bits = VIC2 + 27
  .label multicolor_bits = VIC2 + 28
  .label pointers = screen_memory + 1024 - 8
}

.const SPRITE_BITMAPS = 255-24

.macro set_sprite_position_x(index, position_x) {
  lda #position_x
  sta sprites.positions + index*2 + 0
  lda sprites.position_x_high_bits
  .if (position_x > 255) {
    ora #1 << index
  } else {
    and #255 - [1 << index]
  }
  sta sprites.position_x_high_bits
}

main:
  // initialize music
  jsr music

  lda #%11111111
  sta sprites.enable_bits 

  .for (var i = 0; i < 8; i++) {
    :set_sprite_position_x(i, 60 + 32 * i)
  }

  lda #147
  jsr chrout

  // Disable IRQ handling
  sei 

  // Disable the interrupt generated by timers
  lda #%01111111
  sta cia1_interrupt_control_register
  sta cia2_interrupt_control_register  

  // Negate already scheduled interrupts 
  lda cia1_interrupt_control_register
  lda cia2_interrupt_control_register

  // enable raster interrupts
  lda #%00000001
  sta vic2_interrupt_control_register

  :request_raster_irq(106, raster_bars_irq)
    // Enable IRQ handling
  cli

  loop: jmp loop

sprite_row_irq1:
  :border(RED)
  :sprites_row(70, 8, 3)
  :border(GRAY)

  :request_raster_irq(106, raster_bars_irq)
  lda #%00000001
  sta vic2_interrupt_status_register
  jmp irq_handler

  
raster_bars_irq:
  :border(ORANGE)
  
  :nops(7)
  :background(BLACK)
  :nops(4)
  :background(DARK_GRAY)
  :nops(29)
  :background(GRAY)
  :nops(29)
  :background(LIGHT_GRAY)
  :nops(29)
  :background(WHITE)
  :nops(29)
  :background(LIGHT_GRAY)
  :nops(29)
  :background(GRAY)
  :nops(29)
  :background(DARK_GRAY)
  :nops(29)
  :background(BLACK)
  :nops(4)
  :background(DARK_GRAY)
 
  :border(GRAY)

  :request_raster_irq(150, sprite_row_irq2)
  :exit_irq_handler()

sprite_row_irq2:
  :border(CYAN)
  :sprites_row(160, 16, 12)

  :set_bits(screen_control_register, %00001000)

  :border(GRAY)

  :request_raster_irq(190, play_music)
  :exit_irq_handler()

play_music:
  :border(YELLOW)
  jsr $1003 
  :border(GRAY)
  
  :request_raster_irq(248, open_border_irq)
  :exit_irq_handler()

open_border_irq:
  :border(GREEN)

  :clear_bits(screen_control_register, %00001000)

  :border(GRAY)

  :request_raster_irq(20, sprite_row_irq3)
  :exit_irq_handler()
  
sprite_row_irq3:
  :border(PURPLE)
  :sprites_row(25, 0, 1)
  :border(GRAY)

  :request_raster_irq(60, sprite_row_irq1)
  :exit_irq_handler()

.macro set_bits(address, bit_mask) {
  lda address
  ora #bit_mask
  sta address
}
.macro clear_bits(address, bit_mask) {
  lda address
  and #255 - bit_mask
  sta address
}

.macro border(color) {
  /*lda #color*/
  lda #GRAY
  sta border_color
}
.macro background(color) {
  lda #color
  sta background_color
}
.macro nops(count) {
  .for (var i = 0; i < count; i++) {
    nop
  } 
}

.macro request_raster_irq(line_number, handler) {
  lda #line_number 
  sta vic2_rasterline_register
  
  .if (line_number > 255) {
    :set_bits(screen_control_register, %10000000)
  } else {
    :clear_bits(screen_control_register, %10000000)
  }

  lda #<handler
  sta irq_vector
  lda #>handler
  sta irq_vector + 1
}

.macro sprites_row(y_position, bitmap_offset, color_offset) {
    lda #y_position
  .for (var i = 0; i < 8; i++) {
    sta sprites.positions + 2*i + 1
  }
  .for (var i = 0; i < 8; i++) {
    lda #SPRITE_BITMAPS + bitmap_offset + i
    sta sprites.pointers + i
    lda #color_offset + i
    sta sprites.colors + i
  }
}

.macro exit_irq_handler() {
  lda #%00000001
  sta vic2_interrupt_status_register
  jmp irq_handler
}

.pc=$1000 "Music"
music:
.import binary "music.bin"

.pc = SPRITE_BITMAPS*64 "Sprites"
.import binary "numbers.bin"

.pc = $3fff "Border pattern"
.byte %10110101
